<?php
/**
 * @file
 * Admin configuration page for fancy_grid.
 *
 */

/**
 * Admin configuration page
 */
function fancy_grid_admin($form, &$form_state) {
  $form = array();

  $layouts = _fancy_grid_get_layouts();
  $grid_width = _fancy_grid_get_grid_width();
  $grid_max_width = _fancy_grid_get_grid_max_width();
  $compile = _fancy_grid_get_compile();
  $compile_webservice = _fancy_grid_get_compile_webservice();;

  $form_values = array();
  if (isset($form_state['values']['layouts']))
    $form_values = $form_state['values']['layouts'];

  $default_tab = '';

  // Add new layouts.
  $new_layouts = 0;

  if (isset($form_state['values']['new_layouts'])) {
    // Read all new layouts.
    $new_layouts = $form_state['values']['new_layouts'];

    // Add Breakpoint button was clicked
    if (isset($form_state['clicked_button']) && $form_state['clicked_button']['#name'] == 'add_layout') {
      $new_layouts++;
    }
  }
  elseif (count($layouts) == 0) {
    // Add first layout.
    $new_layouts = 1;
  }

  // Add new layouts to temp config
  for ($i=0; $i < $new_layouts; $i++) {
    $layouts['new' . $i] = array(
      'new' => TRUE,
      'title' => '',
      'machine_name' => '',
      'columns' => 12,
      'gutters' => '2%',
      'breakpoint' => 0,
      'config' => '*|*'
    );
    $default_tab = 'edit-layouts-new' . $i;
  }

  // Remove layout.
  if (isset($form_state['clicked_button']) && strpos($form_state['clicked_button']['#name'], 'remove_') === 0) {
    if (count($layouts) == 1) {
      form_set_error('layout_delete', t('You need at least one layout.'));
    }
    else {
      $deleted_layout = substr($form_state['clicked_button']['#name'], 7);
      unset($layouts[$deleted_layout]);
    }
  }

  $form['new_layouts'] = array(
    '#type' => 'value',
    '#value' => $new_layouts
  );

  // Auto compiling
  $form['compile'] = array(
    '#type' => 'fieldset',
    '#title' => t('Should Fancy Grid compile the css automatically after you save this form?'),
  );

  $options = array(1 => t('Compile via webservice'), 0 => t('I will take care about that myself'));
  $form['compile']['compile'] = array(
    '#type' => 'radios',
    '#default_value' => $compile,
    '#options' => $options,
  );
  $form['compile']['compile_webservice'] = array(
    '#type' => 'textfield',
    '#title' => t('Url to the SassMeister instance for css compilation.'),
    '#description' => t('You can set up your own using <a href="https://www.heroku.com">Heroku</a>. Just clone the code from the <a href="https://github.com/jedfoster/SassMeister">SassMeister repository</a>.'),
    '#default_value' => $compile_webservice,
    '#states' => array(
      'disabled' => array(
        ':input[name="compile"]' => array('value' => 0),
      ),
    ),
  );

  // How to
  $form['compile']['howto'] = array(
    '#type' => 'fieldset',
    '#title' => t('How to implement Fancy Grid in my own sass environment?'),
    '#collapsible' => TRUE,
    '#collapsed' => TRUE,
  );

  $form['compile']['howto']['implementation'] = array(
    '#markup' => '
    <ol>
      <li>
        Require Singularity.gs and Breakpoint in your config.rb file.
<pre>
require \'singularitygs\'
require \'breakpoint\'
</pre>
      </li>
      <li>
        Add Fancy Grid directory as import path to your config.rb file. These may fit your setup:
<pre>
# Fancy Grid imports.
' . implode("\n", _fancy_grid_sass_import_paths()) . '
</pre>
      </li>
      <li>
        Import Singularity.gs and Fancy Grid files in your sass files:
<pre>
@import \'singularitygs\';
@import \'fancy_grid\';
// You can alter all singularitygs configurations between these two imports.
@import \'fancy_grid.utilities\';
</pre>
      </li>
    </ol>
    ',
  );

  $form['grid_width'] = array(
    '#type' => 'textfield',
    '#title' => t('Grid width'),
    '#default_value' => $grid_width,
    '#description' => t('The relative width of the grid. Can be any value. For Example use "90%"" for responsive layouts, "980px" for fixed layouts.'),
    '#required' => TRUE,
  );

  $form['grid_max_width'] = array(
    '#type' => 'textfield',
    '#title' => t('Maximum grid width'),
    '#default_value' => $grid_max_width,
    '#description' => t('The maximum width of the grid. For Example use "980px" for responsive layouts, "none" for fixed layouts.'),
    '#required' => TRUE,
  );

  $form['from'] = array(
    '#type' => 'item',
    '#title' => t('Fancy Grid layout configuration'),
    '#description' => t('These settings will generate the configuration for the
      singularitygs and the Fancy Grid forms.'),
  );

  // Breakpoints configuration.
  $form['layouts'] = array(
    '#type' => 'vertical_tabs',
    '#prefix' => '<div id="layouts-wrapper">',
    '#suffix' => '</div>',
    '#tree' => TRUE,
    '#default_tab' => $default_tab, // This seems not to work with ajax
  );

  foreach ($layouts as $machine_name => $layout) {

    $tab_title = $layout['title'];
    if ($layout['new']) {
      $tab_title = t('New layout');
      if (isset($form_values[$machine_name]['title'])) {
        $tab_title .= ': ' . $form_values[$machine_name]['title'];
      }
    }

    $form['layouts'][$machine_name] = array(
      '#type' => 'fieldset',
      '#title' => $tab_title,
    );

    $default_value = isset($form_values[$machine_name]['title']) ? $form_values[$machine_name]['title'] : $layout['title'];
    $form['layouts'][$machine_name]['title'] = array(
      '#type' => 'textfield',
      '#title' => t('Title'),
      '#default_value' => $default_value,
      '#description' => t('Breakpoint title. Only used on admin pages.'),
      '#required' => TRUE,
    );

    $default_value = isset($form_values[$machine_name]['machine_name']) ? $form_values[$machine_name]['machine_name'] : $layout['machine_name'];
    $form['layouts'][$machine_name]['machine_name'] = array(
      '#type' => 'machine_name',
      '#title' => t('Machine name'),
      '#default_value' => $default_value,
      '#description' => t('Will be used as internal identifier and for css classes. Cannot be changed after creation.'),
      '#disabled' => $layout['machine_name'],
      '#machine_name' => array(
        'exists' => 'fancy_grid_check_machine_name',
      ),
    );

    $default_value = isset($form_values[$machine_name]['columns']) ? $form_values[$machine_name]['columns'] : $layout['columns'];
    $form['layouts'][$machine_name]['columns'] = array(
      '#type' => 'numberfield',
      '#title' => t('Columns'),
      '#default_value' => $default_value,
      '#description' => t('Define the number of columns for this layout.'),
      '#min' => 1
    );

    $default_value = isset($form_values[$machine_name]['gutters']) ? $form_values[$machine_name]['gutters'] : $layout['gutters'];
    $form['layouts'][$machine_name]['gutters'] = array(
      '#type' => 'textfield',
      '#title' => t('Gutters'),
      '#default_value' => $default_value,
      '#description' => t('Define the gutter for this layout. You should <strong>avoid</strong> pixel values or the background-grid debugging will break the sass compiling. Read this <a href="@github-issue">github issue</a> for more details.', array('@github-issue' => 'https://github.com/Team-Sass/Singularity/issues/187')),
    );

    $default_value = isset($form_values[$machine_name]['breakpoint']) ? $form_values[$machine_name]['breakpoint'] : $layout['breakpoint'];
    $form['layouts'][$machine_name]['breakpoint'] = array(
      '#type' => 'numberfield',
      '#title' => t('Breakpoint width'),
      '#default_value' => $default_value,
      '#description' => t('Define at which device with in px the breakpoint will be. Breakpoints with 0 as width will represent the default grid for mobiles.'),
      '#min' => 0,
      '#field_suffix' => 'px'
    );

    $default_value = isset($form_values[$machine_name]['config']) ? $form_values[$machine_name]['config'] : $layout['config'];
    $form['layouts'][$machine_name]['config'] = array(
      '#type' => 'textarea',
      '#title' => t('Class config'),
      '#default_value' => $default_value,
      '#required' => TRUE,
      '#description' => t('Available columns and element ranges for this layout. Format: columns|ranges. Supports * and comma separated values.'),
    );

    // delete button
    $form['layouts'][$machine_name]['remove_layout'] = array(
      '#type' => 'button',
      '#value' => t('Remove this layout'),
      '#name' => 'remove_' . $machine_name,
      '#ajax' => array(
        'callback' => 'fancy_grid_admin_ajax',
        'wrapper' => 'layouts-wrapper',
        'method' => 'replace',
        'effect' => 'fade',
      ),
    );
  }

  $form['add_layout'] = array(
    '#type' => 'button',
    '#value' => t('Add new layout'),
    '#name' => 'add_layout',
    '#ajax' => array(
      'callback' => 'fancy_grid_admin_ajax',
      'wrapper' => 'layouts-wrapper',
      'method' => 'replace',
      'effect' => 'fade',
    ),
  );

  $form['actions']['#type'] = 'actions';

  $form['actions']['force_recompile'] = array(
    '#type' =>'checkbox',
    '#title' => t('Force recompile after successful save.'),
    '#default_value' => isset($form_values['force_recompile']) ? $form_values['force_recompile'] : FALSE,
  );

  $form['actions']['submit'] = array(
    '#type' => 'submit',
    '#value' => t('Save configuration'),
  );

  return $form;
}


/**
 * Ensure unique layout machine name
 */
function fancy_grid_check_machine_name($machine_name) {
  $layouts = variable_get('fancy_grid_layouts', array());

  foreach ($layouts as $layout) {
    if ($layout['machine_name'] == $machine_name) {
      return TRUE;
    }
  }
  return FALSE;
}

/**
 * Implements hook_validate().
 */
function fancy_grid_admin_validate($form, &$form_state) {
  // Remove form errors if we delete a layout.
  if (strpos($form_state['clicked_button']['#name'], 'remove_') === 0) {
    // form_clear_error(); this does not work as expected. Errors messages are still displayed.
  }
}

/**
 * Implements hook_submit().
 */
function fancy_grid_admin_submit($form, &$form_state) {

  // Set config variables
  variable_set('fancy_grid_grid_width', $form_state['values']['grid_width']);
  variable_set('fancy_grid_grid_max_width', $form_state['values']['grid_max_width']);
  variable_set('fancy_grid_compile', $form_state['values']['compile']);
  variable_set('fancy_grid_compile_webservice', $form_state['values']['compile_webservice']);

  unset($form_state['values']['layouts']['layouts__active_tab']);

  // Sort layouts by breakpoint width.
  usort($form_state['values']['layouts'], function($a, $b) {
    return $a['breakpoint'] - $b['breakpoint'];
  });

  // Store the layouts in the configuration array.
  $layouts = array();
  foreach ($form_state['values']['layouts'] as $machine_name => $layout) {
    if (! is_array($layout))
      continue;
    $layouts[$layout['machine_name']] = array(
      'title' => $layout['title'],
      'machine_name' => $layout['machine_name'],
      'columns' => $layout['columns'],
      'gutters' => $layout['gutters'],
      'breakpoint' => $layout['breakpoint'],
      'config' => $layout['config'],
      'new' => FALSE,
    );
  }

  variable_set('fancy_grid_layouts', $layouts);

  $force_compile = FALSE;
  if ($form_state['values']['force_recompile']) {
    $force_compile = TRUE;
  }

  _fancy_grid_write_config_file($force_compile);
}

/**
 * Returns media field wrapper for ajax request
 */
function fancy_grid_admin_ajax($form, $form_state) {
  return $form['layouts'];
}
